# Procédure d'installation

### Accès au projet

Générez une clé SSH sur la machine qui sera votre serveur via la commande `ssh-keygen`. Installez-la, au besoin, via la procédure indiquée sur Internet pour votre OS.

Rendez-vous sur GitLab. Allez dans votre profil, onglet "SSH keys".

Ajoutez votre clé publique avec le bouton associé.

### Clonage du projet

Rendez-vous sur la machine qui sera votre serveur, dans le dossier de votre choix.

Clonez les deux repos Git dans ce dossier, via :

```bash
git clone git@git.unistra.fr:jagermasters/gyokeres-web-rebirth.git
git clone git@git.unistra.fr:jagermasters/gyokeres-api-rebirth.git
```

### Installation de Docker

Via la documentation officielle, installez [Docker Engine](https://docs.docker.com/engine/install/) et [Docker Compose](https://docs.docker.com/compose/install/).

### Création du réseau inter-services

Créez le network permettant à tous les services et au proxy de communiquer ensemble, via :

```bash
docker network create the_all_knowing
```

### Initialisation du proxy (optionnel)

Si vous souhaitez choisir le nom de domaine à utiliser et/ou simplement sécuriser l'accès à votre machine par les utilisateurs du site, récupérez le fichier `compose.yml` sur le [repo Git d'aide au déploiement](https://git.unistra.fr/jagermasters/gyokeres-deploy-assistance).

Rangez-le dans le dossier de votre choix, puis lancez le Nginx Proxy Manager (NPM) via :

```bash
docker compose up -d
```

Rendez-vous sur [l'interface de NPM](localhost:81).

Par défaut, l'email est `admin@example.com` et le mot de passe est `changeme`. Vous pouvez ensuite choisir celui que vous voulez.

Cliquez sur "Proxy host" puis "Add proxy host" et entrez les informations suivantes : !config_proxy_host1 !config_proxy_host2.  
_Les noms de domaine demandés peuvent soit être votre IP (si vous possédez une IP privée ou que vous ne voulez pas de nom de domaine), soit le nom que vous avez acheté au préalable (à OVH, Cloudflare ou autre)._

Si vous possédez une IP publique, vous pouvez aussi demander à Nginx la création d'un certificat SSL (qui crypte les échanges sur le réseau pour plus de sécurité), aussi parfois nommé "le petit cadenas sur Google", via :

!ssl

### Initialisation du projet

Dernière ligne droite !

Rendez-vous là où vous avez cloné le projet (web et API) et, dans chacun des dossiers, lancez les containers via :

```bash
docker-compose -f docker-compose.prod.yml up -d
```

Pour vérifier si les conteneurs sont bien lancés, effectuer la commande

```
docker ps
```

### Troubleshooting

Les conteneurs ne démarrent pas après docker-compose up -d :

- Vérifier les logs avec docker-compose logs -f
- Vérifier si les ports sont déjà utilisés (netstat -tulnp | grep LISTEN)
- Vérifier que docker et docker-compose sont bien installés

Problème de permissions en executant docker-compose up :

- Effectuer la commande

```
sudo usermod -aG docker $USER
```

- Se déconnecter puis se reconnecter

# Prise en main

## Outils de déploiement

Lors de la réception du projet, nous n'avons pas trouvé de procédure d'installation. Nous avons donc peu parcouru les différents projets, et avons trouvé deux fichiers docker-compose permettant de déployer respectivement l'API et l'application Web (comme précisé dans la documentation d'installation ci dessus). Nous avons donc repris ces fichiers pour déployer les applications.

Nous n'avons également pas trouvé de foire aux questions ni de troubleshooting.

Le pipeline est fonctionnelle, mais a necessité la modification de l'adresse de déploiement sans trop de surprise (la modification a été très rapide).

Les script init.sh faisaient la même chose que les docker-compose avec quelques fantaisie sur le choix d'environnement. Environnement qui utilisait beaucoup de variables externes (.env), pas forcément nécessaire. Nous avons simplifié en enlevant lesdits scripts (transféré vers les dockerfiles) et en écrémant les variables en trop.

Nous avons également mis à jour les containers pour qu'aucun n'expose ses ports volontairement, pour des raisons de sécurité.

## Technologies

L'api utilise des technos vues en cours (prisma, express, jwt, etc) ou utilisées dans nos anciens projet (swagger, etc). L'extension et la maintanbilité ne devraient pas poser trop de problèmes.

## Ancien protocole d'installation

Au besoin, les bribes de documentation et infos transmises par l'ancien groupe en charge du développement de Gyokereske sont disponibles [ici](https://git.unistra.fr/jagermasters/gyokeres-deploy-assistance/-/blob/main/Ancien%20protocole.md?ref_type=heads). Nous y avons ajouté un guide de création d'APK pour distribuer l'application mobile.

## Problèmes

### Problèmes techniques

- Joueur parfois kické des quiz de manière random après une question (quand plusieurs ont la même bonne réponse en même temps, on dirait)
- QR code envoie vers localhost
- Parfois, l'application fail de manière aléatoire
- Modes multi ont une connexion instable
- Modes multi sont affreusement lents (problème d'ergonomie, long à redévelopper)
- Dans la CI/CD, les tests et le lint ne marchent pas, dans le sens où ils sont mal configurés et il manque des dépendances

### Problèmes UI/UX

- Interface web pas responsive
- Clignotement du "Back to Home"
- L'interface pour rejoindre est nulle (aucun style CSS)
- Sign-up : si jamais on crée un compte avec un nom déjà utilisé, ça nous dit juste "Création du compte impossible" ou "Invalid credentials"
- Le style de la modal pour rejoindre est affreux, c'est du natif HTML sans CSS

### Problèmes liés aux ressources et contenus

- Logo sous licence
- Il n'y a pas de gestion des médias

### Problèmes divers

- Une note beaucoup trop haute pour un projet à moitié terminé

## Interactions inter-équipes

Deux groupes sont venus nous redemander un petit topo sur mon ancien projet, Kwizos, car la doc n'était pas à jour par manque de temps. Je leur ai fourni des instructions simplifiées et à jour (non pushées pour rester dans la logique du RP) expliquant comment j'avais réussi à déployer le projet.

Un groupe est revenu me voir quelque temps plus tard, car les instructions étaient erronées... Sauf qu'en discutant plus longuement avec eux, j'ai bien vu que les membres :

- n'ont soit pas lu les instructions, préférant RE-demander des choses déjà dites dans une doc pourtant concise (car l'ancienne doc était aussi trop longue selon eux)
- ont choisi une branche aléatoire pour déployer (pourquoi ?)
- revenaient me voir pour me demander si telle ou telle feature était en place, pour que je leur indique que "oui, comme c'est visible dans l'interface", avant de voir qu'ils n'ont même pas essayé de lancer...
- me demandaient pourquoi il n'y a pas de script pour lancer facilement l'application entière (ex : un `init.sh`), alors que nous avons suivi la logique de `docker-compose`, qui le fait déjà à votre place en une seule commande simple, à la fois dispo sur notre doc, sur la documentation officielle (et ayant été vue en cours si on sort du RP).

En tant que membre ayant écrit plus de 60% du projet _Kwizos by The Sevens_, j'ai bien conscience que certaines fonctionnalités sont instables... Contrairement au déploiement, qui a fonctionné sur plusieurs machines différentes au fil du projet, de manière efficace, puisque nous avons dû changer trois fois d'hébergement de serveur (j'ai même dû le remanier plusieurs fois pour pouvoir le lancer sur ma propre petite Raspberry Pi).

Je suis juste surpris par les bruits de couloir, les groupes qui râlent sans demander d'indications pour autant, et je m'inquiète du manque d'interaction avec les membres de _The Sevens_ alors que TOUS _devraient être_ disponibles pour répondre aux questions, _pas seulement moi_.

Il faut donc avouer que, de mon point de vue, la situation est telle qu'une partie des groupes ayant reçu Kwizos part du principe que "le projet n'a pas l'air bien à première vue, ils n'ont pas eu une bonne note, alors je ne fais rien et renverrai la faute sur les autres". Là où l'autre moitié tente des choses... extravagantes (même si je ne saurais dire s'ils le font pour le roleplay ou non).

J'ai déjà subi la douloureuse note de ce projet lors du développement, j'aurais tout fait pour aider les membres des autres groupes (sans pour autant leur faire 100% du travail). Donc je voulais mettre en garde ici, pour la correction, afin de ne pas entrer dans un cercle vicieux non mérité.

Enfin, et cette fois pour être plus taquin, je précise que le groupe qui nous a le plus critiqués de face, en tout bien tout honneur, a lui aussi des problèmes avec son ancien projet (aucune documentation, script modifiant les configurations Docker bloquant des projets pourtant indépendants, non-respect de beaucoup de conventions vues en cours et présentes en entreprise, ...). Or là, les gens (nous et les autres groupes l'ayant reçu) essayent et réussissent à déployer !

-- Samuel

On n'est pas venu me demander d'aide pour déployer mon ancienne application (Mogula). On a plutôt demandé a Mathieu Ackermann, qui s'est occupé intégralement du déploiement en collaboration avec Louis Schlegel.

-- Eliott

## Configuration de Docker

Le script de lancement du groupe Gyokeres apporte une configuration spécifique à Docker tout en "forçant" l'utilisateur à être sur une machine Linux.

Autant la deuxième anomalie est réglable en précisant une image Docker basée sur Linux (ex : les versions "Alpine" des images X ou Y), autant la première a perturbé le projet d'alternance et provoqué plusieurs heures de debug sur un projet qui n'avait pourtant rien à voir...

# FAQ

## Existait-il une procédure d'installation dans le projet que vous avez récupéré ?

Non, mais une procédure est facilement "devinable".

## Cette procédure était-elle à jour ou contenait-elle des erreurs ?

Les scripts liés à cette "procédure" était en effet à jour, sans bug, mais avec quelques anomalies.

## Avez-vous modifié cette procédure pour la rendre plus cohérente et consistante avec les étapes que vous avez réalisées ?

Nous avons modifié la procédure afin d'utiliser au maximum docker compose et plus de ressource (script shell etc) à côté ainsi que simplifié la gestion d'environnements (variables, choix spécifiques en fonctionnemen de l'environnement mais qui ne change rien au résultat).

## Avez-vous changé volontairement de technologies pour parvenir à redéployer l'ensemble du projet ? Si oui, pourquoi ?

Nous avons juste ajouté nginx pour la sécurité et pouvoir accéder au site via un nom de domaine que nous possédions, a contrario de l'ancien proposé.

## Une foire aux question ou un document type Troubleshooting est-il présent dans le projet récupéré ?

Non.

## Avez-vous rencontré des difficultés non signalées dans le projet récupéré lors de son installation ?

Oui, au bout de quelques jours, nous avons remarqué une erreur "d'api impossible à atteindre". Il savère, qu'en fait, l'url de l'api était codée en dur et communiquait non pas avec l'api locale mais celle du site gyokereske.redirectme.net ! Site maintenant cloturé.

## Le pipeline d'intégration continue et de déploiement continu est-il toujours fonctionnel ? A-t-il besoin de modifications et si oui, que faudrait-il faire pour le remettre en état et combien de temps estimez-vous nécessaire sa remise en place ?

Il faut simplement changer deux points importants :

- le nom d'utilisateur et ip, codés en dur pour l'ancien site, qu'on met en variable "d'environnement" et qu'on change avec les notres
- les codntions de lancement : à chaque push, les pipelines des différents repo sont lancées, nous les limitons à "main"
